########################################
# ts.tcl
########################################

# Procedures

proc debug {s} {
  global params

  if $params(debug) { puts $s }
}

proc filetest {f1 {f2 ""}} {
  if ![file exist $f1] {
    return 1
  }
  if {$f2 == ""} {return 0}
  if ![file exist $f2] {
    return 2
  }
  if {[file mtime $f1] > [file mtime $f2]} {
    return 3
  }
  return 0
}

# proc scanopt replaces  %-parameters
#  %p - Primary file 
#  %r - Rootname of Primary file
#  %e - Editfile -> Filedialog
#  %o - Other file -> Filedialog
#  %g - Geometry 
# Print options !... ! cleared

proc scanopt {s {geom ""}} {
  global params prd

  debug "scanopt: $s"
  set t ""
  if [regexp -- {%p} $s] {
    if {$params(Primary_file) != ""} { 
      set t $params(Primary_file)
    } elseif {$params(Edit_file) != ""} {
      set t [cutdir $params(Edit_file)]
    } else {
      Warning $params(dlg_geom) \
        "No Primary file is specified.\nNo Edit file was found."
      return ""
    }
    regsub -all -- {%p} $s $t s
    debug $s
  }
  if [regexp -- {%r} $s] {
    if {$params(Primary_file) != ""} { 
      set t [file rootname $params(Primary_file)]
    } elseif {$params(Edit_file) != ""} {
      set t [file rootname [cutdir $params(Edit_file)]]
    } else {
      Warning $params(dlg_geom) \
        "No Primary file is specified.\nNo Edit file was found."
      return ""
    }
    regsub -all -- {%r} $s $t s
    debug $s
  }
  regsub -all -- {%e} $s $params(Edit_file) s
  debug $s
  if [regexp -- {%o([^ ]*)} $s sub ext] {
    set f ""
    if {![FileDialog $params(dlg_geom) "Select File" $ext f]} {
      return ""
    }
    if {[file tail $f] != ""} {
      set t [cutdir $f]
    }
    regsub -all -- {%o[^ ]*} $s $t s
    debug $s
  }
  if [regexp -- {%g([^ ]*)} $s sub sgeo] {
    if {$sgeo != ""} {
      set geom $sgeo
    }
    if {$geom == ""} {
      set geom $params(default_geom)
    }
    regsub -all -- {%g[^ ]*} $s $geom s
    debug $s
  }
  regsub -all -- {![^!]*!} $s {} s
  debug "scanopt ready: $s"
  return $s
}

# setprintopt s opt
# sets printeroption opt in s
# Options:
#   landscape -> !l..!
#
proc setprintopt {str opt {value ""}} {
  upvar $str s

  debug "Setprintopt ($opt $value): $s"
  switch $opt {
    landscape {
      regsub {!l([^!%]*)!} $s "\\1" s
    }
    first {
      regsub {!0([^!%]*)%([^!]*)!} $s "\\1$value\\2" s
    }
    last {
      regsub {!9([^!%]*)%([^!]*)!} $s "\\1$value\\2" s
    }
    copies {
      regsub {!c([^!%]*)%([^!]*)!} $s "\\1$value\\2" s
    }
    odd {
      regsub {!o([^!%]*)!} $s "\\1" s
    }
    even {
      regsub {!e([^!%]*)!} $s "\\1" s
    }
    xopt {
      regsub {!x([^!%]*)%([^!]*)!} $s "\\1$value\\2" s
    }
    default {return 0}
  }
  debug "Setprintopt ready: $s"
  return 1
}

proc EvalCmd {cmdline {geom ""}} {
  global params sigchld

  debug "EvalCmd: $cmdline"
  set cmd [lindex $cmdline 1]
  set opt [lindex $cmdline 2]
  if {[set opt [scanopt $opt $geom]] == ""} {
    return
  }
  debug "Work directory: $params(Primary_dir)"
  debug "Cmd: $cmd Opt: $opt"
  switch $cmd {
    exwin {
      OutMsg "$opt\n"
      eval ExecCmd $opt
      set oldfocus [focus]
      debug "exwin: oldfocus = $oldfocus"
      xfocus .term.t
      tkwait variable sigchld
      debug "exwin: process died"
      xfocus $oldfocus
      update idletasks
    }
    exbg {
      eval ExecBg $opt
    }
    exec {
      OutMsg $opt
      catch {eval exec $opt} out
      OutMsg $out
    }
    default {
      OutMsg "$cmd $opt"
      eval $cmd $opt
    }
  }
}

proc OutMsg {s} {
  Out .t.text "+ [CurTime] $s"
}

proc OutErr {s} {
  Out .t.text  "??? $s ???\n"
}

proc OutClr {} {
  Out .t.text \012
}

proc xfocus {w} {
  global params ted

  set old_focus [focus]
  if {$old_focus == ".term.t"} {
    catch {$old_focus configure -foreground #9d9d9d}
  }
  if {[string compare $w ""] == 0} {set w .}
  focus $w
  raise [set wt [winfo toplevel $w]]
  if {$w == ".term.t"} {
    catch {$w configure -foreground #000000}
  }
  catch {set params(Edit_file) $ted($wt,file)}
}

proc Exit {} {
  global params env

  set ex_ok 1
  for {set i 0} {$i < 10} {incr i} {
    if [winfo exists .t$i] {
      if ![TedExit .t$i] {set ex_ok 0}
    }
  }
  if {!$ex_ok} {
    return
  }
  writeparams $env(HOME)/.texshell
#PROF  profile off prof
#PROF  profrep prof cpu ts.profile
  CleanShutdown
  exit
}

proc Primary {} {
  global params

  if {$params(Primary_file) != ""} {
    set f $params(Primary_file)
  } else {
    set f ""
  }
  if {![FileDialog $params(dlg_geom) "Primary File" .tex f]} {return 0}
  set params(Primary_dir) [file dirname $f]
  cd $params(Primary_dir)
  set f [file tail $f]
#  set params(Primary_file) [file rootname $f]
  set params(Primary_file) $f
  OutMsg "Primary file was changed to $params(Primary_file)"
  OutMsg "Working directory is now    $params(Primary_dir)"
  return 1
}

proc EditFile {file {line 1}} {
  global params ted

  set lcmd [lindex $params(Edit) $params(Edit_default)]
  set cmd [lindex $lcmd 1]
  if {$cmd == "tedit"} {
    for {set i 0} {$i < 10} {incr i} {
      if [winfo exists .t$i] {
        if {$ted(.t$i,file) == $file} {
          raise .t$i
          xfocus .t$i.text
          .t$i.text mark set insert $line.0
          .t$i.text yview -pickplace insert
          return
        }
      }
    }
    set i 1
    while {[winfo exists .t$i]} {incr i}
    if {$i >= 10} {
      Warning $params(dlg_default) "Too many editors (max. 9)."
      return
    }
    set geom +[expr $params(x0) + 50 + 30 * $i]+[expr 50 + 30 * $i]
    tedit .t$i $geom "Edit - $i" $file $line
  } else {
    set opt [lindex $lcmd 2]
    regsub -- {!i([^!%]*)%([^!]*)!} $opt "\\1$line\\2" opt
    debug "Insert line: $opt"
    set params(Edit_file) $file
#    if {[set i [string first "%e" $opt]] >= 0} {
#      set s [string range $opt 0 [expr $i - 1]]$file
#      set l [string length $opt]
#      for {incr i 2} {$i < $l && [cindex $opt $i] != " "} {incr i} { }
#      set opt $s[string range $opt [expr $i + 1] end]
#    } else {
#      Warning $params(dlg_geom) \
#        "Error in Editor file parameter.\nMust be `%e'."
#      return
#    }
    EvalCmd [list Edit $cmd $opt]
  } 
}
      
proc Edit {{i ""}} {
  global params

  set lcmd [lindex $params(Edit) $params(Edit_default)]
  set cmd [lindex $lcmd 1]
  # eval geometry for external Editor
  if {$cmd != "tedit"} {
    set ext [file extension $params(Edit_file)]
    if {$ext == ""} {set ext .tex}
    if {![FileDialog $params(dlg_geom) "Select File" $ext \
      params(Edit_file)]} {return ""}
    set params(Edit_file) [cutdir $params(Edit_file)]
    EvalCmd $lcmd
    return
  }
  if {$i != ""} {
    if {[winfo exists .t$i]} {
      raise .t$i
      xfocus .t${i}.text
      return
    }
  } else {
    set i 1
    while {[winfo exists .t$i]} {incr i}
    if {$i >= 10} {
      Warning $params(dlg_default) "Too many editors (max. 9)."
      return
    }
  }
  set geom +[expr $params(x0) + 50 + 30 * $i]+[expr 50 + 30 * $i]
  if {![FileDialog $params(dlg_geom) "Select File" .tex \
    params(Edit_file)]} {return ""}
  set opt ".t$i $geom \"Edit - $i\" $params(Edit_file)"
  if {$opt != ""} {eval tedit $opt} 
}

proc EditPrim {} {
  global params
  
  set lcmd [lindex $params(Edit) $params(Edit_default)]
  set cmd [lindex $lcmd 1]
  if {$params(Primary_file) == ""} {
    if {![Primary]} return
    if {$params(Primary_file) == ""} {
      Warning $params(dlg_geom) "No primary file is specified."
      return
    }
  }
  if {[file extension $params(Primary_file)] != ""} {
    set params(Edit_File) $params(Primary_file)
  } else {
    set params(Edit_File) $params(Primary_file).tex
  }
  if {$cmd == "tedit"} {
    if {[winfo exists .t0]} {
      raise .t0
      xfocus .t0.text
    } else {
      set x [expr $params(x0)+50]
      tedit .t0 +$x+50 "Edit - 0" \
        $params(Primary_dir)/$params(Primary_file)
    }
  } else {
    EditFile $params(Primary_file) 1
  }
}

proc Compose {} {
  global params

  # Save last edit file, because Saving will change this
  set edfile $params(Edit_file)
  for {set i 0} {$i < 10} {incr i} {
    if [winfo exists .t$i] {
      if {![TedAskSave .t$i]} {return}
    }
  }
  set params(Edit_file) $edfile
  set l [lindex $params(Compose) $params(Compose_default)]
  unlink compose.err
  EvalCmd $l
  if {[file exists compose.err]} { ;# Error occured, Edit was called
    set fd [open compose.err r]
    gets $fd s
    close $fd
    regexp -- {([^ ]*)[ ]*([^ ]*)} $s sub line file
    unlink compose.err
    if {[string first "/" $file] != 0} {
      set file $params(Primary_dir)/$file
    }
    debug "Compose: $file $line"
    EditFile $file $line
  }
}

proc Reference {} {
  global params

  set l [lindex $params(Reference) $params(Reference_default)]
  EvalCmd $l
}

proc View {} {
  Output View
}

proc Print {} {
  Output Print
}

proc Output {{device View}} {
  global params prd

  if {$params(Primary_file) != ""} {
    set f [file rootname $params(Primary_file)]
  } elseif {$params(Edit_file) != ""} {
    set f [file rootname $params(Edit_file)]
  } else {
    Warning $params(dlg_geom) "No Primary file is specified."
    return
  }

  switch [filetest ${f}.tex ${f}.dvi] {
    1 {
      Warning $params(dlg_geom) "TeX file ${f}.tex not found"
      return
    }
    2 {
      if {[Dialog .dlg $params(dlg_geom) "" \
        "DVI file $f.dvi not found.\n Run compose?" question \
        0 1 {Yes <Any-y>} {No <Any-n>}] == 0} {
        Compose
      } else {
        return
      }
    }
    3 {
      if {[Dialog .dlg $params(dlg_geom) "" \
        "Primary file $f has been changed.\nRun compose?" question \
        0 1 {Yes <Any-y>} {No <Any-n>}] == 0} {
        Compose
        debug "View weiter"
      }
    }
  }
  set l [lindex $params($device) $params(${device}_default)]
  # .ps test
  set cmd [lindex $l 1]
  set opt [lindex $l 2]
  if {[string first "%r.ps" $opt] != -1} {
    switch [filetest ${f}.dvi ${f}.ps] {
      1 {
        Warning $params(dlg_geom) "Could not compose DVI file ${f}.dvi."
        return
      }
      default {
        OutMsg "Generate new PostScript file."
        EvalCmd [list MakePS exwin "dvips -o%r.ps %r"]
      }
    }
  } 
  if {$device == "View"} {
    EvalCmd $l
    return
  }
  # Printing
  if {![PrintDlg $params(dlg_geom) [cutdir "${f}.dvi"] params]} {
    return
  }
  if {$params(pr_first) != "" && $params(pr_first) > 1} {
    setprintopt opt first $params(pr_first)
  }
  if {$params(pr_last) != "" && $params(pr_last) < 9999} {
    setprintopt opt last $params(pr_last)
  }
  if {$params(pr_copies) != "" && $params(pr_copies) > 1} {
    setprintopt opt copies $params(pr_copies)
  }
  if {$params(pr_xopt) != ""} {
    setprintopt opt xopt $params(pr_xopt)
  }
  if {$params(pr_orientation) == "landscape"} {
    if {![setprintopt opt landscape]} {
      Warning $params(dlg_geom) \
        "DVI Driver can't print in landscape mode.\nAbort."
      return
    }
  }
  switch $params(pr_pages) {
    all { }
    odd {setprintopt opt odd}
    even {setprintopt opt even}
  }
  set l [list Print $cmd $opt]
  EvalCmd $l
}

proc Graphic {} {
  global params

  set l [lindex $params(Graphic) $params(Graphic_default)]
  EvalCmd $l
}

proc Util {{i 0}} {
  global params

#  set l {}
#  foreach i $params(Util) {
#    lappend l [lindex $i 0]
#  }
#  if {[set i \
#      [ListDlg .ldlg 300x150$params(dlg_geom) "Utilities:" $l \
#        $params(Util_default)] ] > -1} {
    set l [lindex $params(Util) $i]
    EvalCmd $l
#  }
}

proc setparam {cmd entry} {
 global params

 set params(${cmd}_cmd) [lindex $entry 1]
}

proc ShowOut {} {
  global params

  if {$params(ShowOut)} {
    wm deiconify .term
  } else {
    wm withdraw .term
  }
}

# Main
proc main {} {
  global params pid argc argv env hlp

  readparams $env(HOME)/.texshell
  ide
  # Extent standard action for Button 1 (set focus)
  set textbind "[bind Text <1>]\nxfocus %W"
  bind Text <1> $textbind
  debug [bind Text <1>]
  #bind .c.lprimary <2> {set params(Primary_file) [selection get]}

  if $argc>0 {
    set f [lindex $argv 0]
    set params(Primary_dir) [file dirname $f]
    cd $params(Primary_dir)
    set f [file tail $f]
#    set params(Primary_file) [file rootname $f]
    if {[file extension $f] == ""} {
      set params(Primary_file) ${f}.tex
    } else {
      set params(Primary_file) $f
    }
  }
  if {$params(Primary_dir) == "."} {
    set params(Primary_dir) [pwd]
  }
  set env(TEXEDIT) "ts_err %d %s"
  foreach P {Edit Compose View Print Util Graphic} {
    set params(${P}_pid) 1
  }
  debug "Start Help"
#  ref_tell [glob -nocomplain $hlp(dir)/*.hlp]
  ref_read
  debug "Help Ready"
  update
  OutMsg "TeXShell (Jens Poenisch: poenisch@wirtschaft.tu-chemnitz.de)"
  OutMsg "Version $params(version)"
  OutMsg "  Primary file:      $params(Primary_file)"
  OutMsg "  Working directory: $params(Primary_dir)"
  OutMsg "  Help: Press F1"
  OutMsg ""
  OutMsg "Please delete your config file ~/.texshell or replace the"
  OutMsg "command line parameter %p by %r\[.extension\] for all tools"
  OutMsg "working on composed files. See also the help."
  update idletasks
#  focus .term.t
  focus .
  update
} 
